@@ -135,7 +135,8 @@ typedef enum {
   DE265_NON_EXISTING_LT_REFERENCE_CANDIDATE_IN_SLICE_HEADER=1023,
   DE265_WARNING_CANNOT_APPLY_SAO_OUT_OF_MEMORY=1024,
   DE265_WARNING_SPS_MISSING_CANNOT_DECODE_SEI=1025,
-  DE265_WARNING_COLLOCATED_MOTION_VECTOR_OUTSIDE_IMAGE_AREA=1026
+  DE265_WARNING_COLLOCATED_MOTION_VECTOR_OUTSIDE_IMAGE_AREA=1026,
+  DE265_WARNING_PCM_BITDEPTH_TOO_LARGE=1027
 } de265_error;
 
 LIBDE265_API const char* de265_get_error_text(de265_error err);
@@ -186,6 +187,11 @@ LIBDE265_API void de265_get_image_NAL_header(const struct de265_image*,
                                              int* nuh_layer_id,
                                              int* nuh_temporal_id);
 
+LIBDE265_API int de265_get_image_full_range_flag(const struct de265_image*);
+LIBDE265_API int de265_get_image_colour_primaries(const struct de265_image*);
+LIBDE265_API int de265_get_image_transfer_characteristics(const struct de265_image*);
+LIBDE265_API int de265_get_image_matrix_coefficients(const struct de265_image*);
+
 
 /* === decoder === */
 
